import { Button, Form, Input, message, Popconfirm, Table } from 'antd'
import React, { useContext, useEffect, useRef, useState } from 'react'
import styled from 'styled-components'
import { Link, useNavigate } from 'react-router-dom'
import { getJobDataList, deleteJob } from '../services'
const EditableContext = React.createContext(null)
const FormItem = Form.Item

const JobItems = styled.div`
  background-color: #ffffff;
  .delButton {
    margin-left: 40px;
  }
  .actionBtn {
    cursor: pointer;
    color: #1890ff;
  }
  .ant-table-wrapper {
    margin-right: 12px;
    .ant-table {
      margin-left: 16px;
    }
  }
`
const EditableRow = ({ index, ...props }) => {
  const [form] = Form.useForm()
  return (
    <Form form={form} component={false}>
      <EditableContext.Provider value={form}>
        <tr {...props} />
      </EditableContext.Provider>
    </Form>
  )
}

const EditableCell = ({
  title,
  editable,
  children,
  dataIndex,
  record,
  handleSave,
  ...restProps
}) => {
  const [editing, setEditing] = useState(false)
  const inputRef = useRef(null)
  const form = useContext(EditableContext)
  useEffect(() => {
    if (editing) {
      inputRef.current.focus()
    }
  }, [editing])

  const toggleEdit = () => {
    setEditing(!editing)
    form.setFieldsValue({
      [dataIndex]: record[dataIndex],
    })
  }

  const save = async () => {
    try {
      const values = await form.validateFields()
      toggleEdit()
      handleSave({ ...record, ...values })
    } catch (errInfo) {
      console.log('Save failed:', errInfo)
    }
  }

  let childNode = children

  if (editable) {
    childNode = editing ? (
      <FormItem
        style={{
          margin: 0,
        }}
        name={dataIndex}
        rules={[
          {
            required: true,
            message: `${title} is required.`,
          },
        ]}>
        <Input ref={inputRef} onPressEnter={save} onBlur={save} />
      </FormItem>
    ) : (
      <div
        className="editable-cell-value-wrap"
        style={{
          paddingRight: 24,
        }}
        onClick={toggleEdit}>
        {children}
      </div>
    )
  }

  return <td {...restProps}>{childNode}</td>
}

const App = () => {
  const [dataLoading, setDataLoading] = useState(false)
  //获取作业列表数据
  const getJobTables = async () => {
    const project_id = 'test'
    // const project_id = sessionStorage.getItem('project_id')
    setDataLoading(true)
    const { data } = await getJobDataList(project_id)
    if (data.code === 200) {
      const list = data.data.map(item => {
        return {
          key: item.id,
          name: item.name,
          type: item.type,
          tag: item.tag,
          create_time: item.create_time,
          update_time: item.update_time,
        }
      })
      setDataSource(list)
    } else {
      message.error(data.msg)
    }
    setDataLoading(false)
  }

  // componentDidMount
  useEffect(() => {
    getJobTables()
  }, [sessionStorage.getItem('project_id')])
  const [dataSource, setDataSource] = useState()

  const navigate = useNavigate()

  //删除作业
  const handleDelete = async key => {
    const { data } = await deleteJob(key)
    if (data.code === 200) {
      const newData = dataSource.filter(item => item.key !== key)
      setDataSource(newData)
      message.success('删除成功')
    } else {
      message.error('删除失败')
    }
  }

  const editJobitem = id => {
    navigate('/work-management/cre', { state: { id } })
  }

  // 格式化事件
  const formatTime = time => {
    const date = new Date(Number(time) * 1000)
    const YY = date.getFullYear()
    const MM =
      date.getMonth() + 1 < 10
        ? '0' + (date.getMonth() + 1)
        : date.getMonth() + 1
    const DD = date.getDate() < 10 ? '0' + date.getDate() : date.getDate()
    const hh = date.getHours() < 10 ? '0' + date.getHours() : date.getHours()
    const mm =
      date.getMinutes() < 10 ? '0' + date.getMinutes() : date.getMinutes()
    const ss =
      date.getSeconds() < 10 ? '0' + date.getSeconds() : date.getSeconds()
    return YY + '-' + MM + '-' + DD + ' ' + hh + ':' + mm + ':' + ss
  }

  const defaultColumns = [
    {
      title: '作业名称',
      dataIndex: 'name',
      width: '16.6%',
    },
    {
      title: '作业类型',
      dataIndex: 'type',
      width: '16.6%',
    },
    {
      title: '作业分类',
      dataIndex: 'tag',
      width: '16.6%',
    },
    {
      title: '创建时间',
      dataIndex: 'create_time',
      width: '16.6%',
      render: time => formatTime(time),
    },
    {
      title: '更新时间',
      dataIndex: 'update_time',
      width: '16.6%',
      render: time => formatTime(time),
    },
    {
      title: '操作',
      dataIndex: 'operation',
      width: '16.6%',

      render: (_, record) =>
        dataSource.length >= 1 ? (
          <>
            <span className="actionBtn" onClick={() => editJobitem(record.key)}>
              编辑
            </span>

            <Popconfirm
              title="确认删除？"
              okText="确认"
              cancelText="取消"
              onConfirm={() => handleDelete(record.key)}>
              <span className="delButton actionBtn">删除</span>
            </Popconfirm>
          </>
        ) : null,
    },
  ]
  //创建作业

  const handleSave = row => {
    const newData = [...dataSource]
    const index = newData.findIndex(item => row.key === item.key)
    const item = newData[index]
    newData.splice(index, 1, { ...item, ...row })
    setDataSource(newData)
  }

  const components = {
    body: {
      row: EditableRow,
      cell: EditableCell,
    },
  }
  const columns = defaultColumns.map(col => {
    if (!col.editable) {
      return col
    }

    return {
      ...col,
      onCell: record => ({
        record,
        editable: col.editable,
        dataIndex: col.dataIndex,
        title: col.title,
        handleSave,
      }),
    }
  })
  return (
    <JobItems>
      <Link to="/work-management/cre">
        <Button
          type="primary"
          style={{
            float: 'right',
            marginBottom: 16,
            marginTop: '18px',
            marginRight: '16px',
            fontSize: '12px',
          }}>
          创建作业
        </Button>
      </Link>

      <Table
        components={components}
        rowClassName={() => 'editable-row'}
        bordered
        loading={dataLoading}
        dataSource={dataSource}
        columns={columns}
      />
    </JobItems>
  )
}

export default App
